# frozen_string_literal: true

require File.expand_path('../spec_helper', __dir__)

module Danger
  describe Danger::DangerWCC do
    before do
      @dangerfile = testing_dangerfile
      @my_plugin = @dangerfile.wcc
      @git = @dangerfile.git
      @github = @dangerfile.github

      allow(@github).to receive(:pr_json)
        .and_return(JSON.parse(load_fixture('github_pr.json')))
      allow(subject).to receive(:run)
        .with(/npm/)
        .and_return('')
      allow(subject).to receive(:run_and_diff)
        .and_return(load_fixture('yarn_deduplicate/list.diff'))

      allow(File).to receive(:exist?).and_call_original
      allow(File).to receive(:read).and_call_original
      allow(File).to receive(:exist?).with('yarn.lock')
        .and_return(true)
      allow(File).to receive(:readlines).with('yarn.lock')
        .and_return(load_fixture('yarn_deduplicate/yarn.lock').split("\n"))
    end

    describe 'yarn_deduplicate' do
      let(:subject) { Danger::DangerWCC::YarnDeduplicate.new(@my_plugin) }

      it 'runs yarn-deduplicate and parses diff' do
        # act
        subject.perform

        # assert
        warnings = @dangerfile.violation_report[:warnings]
        expect(warnings.length).to eq(2)

        expect(warnings[0].message)
          .to include('You have an opportunity to deduplicate "@babel/core".')
        expect(warnings[0].message)
          .to include('It\'s using 7.4.0 but could use the existing version '\
            '7.12.10.')
        expect(warnings[0].message)
          .to include('npx yarn-deduplicate -s fewer --packages "@babel/core"')
        expect(warnings[0].file).to eq('yarn.lock')
        expect(warnings[0].line).to eq(24)

        expect(warnings[1].message)
          .to include('npx yarn-deduplicate -s fewer --packages "@babel/core"')
        expect(warnings[1].file).to eq('yarn.lock')
        expect(warnings[1].line).to eq(24)
      end
    end
  end
end
